# est-variance-fishing.R, 20 Oct 24

#
# inconsistency_est.R,  9 Sep 17
# Data from:
# Inconsistency of Expert Judgement-based Estimates of Software Development Effort
# Stein Grimstad and Magne J{\o}rgensen
#
# Example from:
# Empirical Software Engineering using R
# Derek M. Jones


library("plyr")
library("reshape2")

par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(6)


plot_subj=function(df)
{
# Relies on each subject being adjacent in df
points(subset(df, Order == 1)$Estimate,
       subset(df, Order == 2)$Estimate, col=pal_col[df$Task[1]], cex=1.4)
}


incon=read.csv("inconsistency_est.csv.xz", as.is=TRUE)

tasks=melt(incon, measure.vars=paste0("T", 1:6),
                variable.name="Task", value.name="Estimate")

plot(0.1, type="n", log="xy", 
        xlim=c(1, 80), ylim=c(1, 80),
        xlab="First estimate (hours)", ylab="Second estimate (hours)")

lines(c(0.5, 100), c(0.5, 100), col="grey")

d_ply(tasks, .(Task), plot_subj)

legend(x="bottomright", legend=paste0("Task ", 1:6), bty="n", fill=pal_col, cex=1.2)

First=subset(tasks, Order == 1)
Second=subset(tasks, Order == 2)


first_est=subset(tasks, Order == 1)$Estimate
second_est=subset(tasks, Order == 2)$Estimate

RE=abs(first_est-second_est)/first_est
mean(RE)


# Fit a deming regression model to  1st/2nd estimates
library("deming")

# Assume equal eror in first/second estimate
#edif_mod=deming(second_est ~ first_est)
edif_mod=deming(log(second_est) ~ log(first_est))
# summary(edif_mod) # not support for deming :-(
coef(edif_mod)
aov(edif_mod) # find the residual standard error



# Now build an errors-in-variables model
# Need support for building errors-in-variables regression models
library("simex")

Sip_all=read.csv("Sip-task-info.csv.xz", as.is=TRUE)

Sip_uTN=subset(Sip_all, !duplicated(TaskNumber)) # Keep one instance of each multi-person task
# Removed the 190 tasks that were cancelled before completion
Sip=subset(Sip_uTN, StatusCode != "CANCELLED")
# Single instance of this in the data
Sip=subset(Sip, StatusCode != "TEMPLATE")
# Projects that were not completed at the time of the data snapshot
Sip=subset(Sip, StatusCode != "CHRONICLE")

plot(Sip$HoursEstimate, Sip$HoursActual, log="xy", col=pal_col[3],
        xlim=c(0.2, 1e3), ylim=c(0.2, 1e3),
        xlab="Estimate (hours)", ylab="Actual (hours)\n")
lines(c(1e-2, 1e3), c(1e-2, 1e3), col="grey")

Sip$lHoursActual=log(Sip$HoursActual)
Sip$lHoursEstimate=log(Sip$HoursEstimate)
Sip_mod=glm(lHoursActual ~ lHoursEstimate, data=Sip, x=TRUE, y=TRUE) # x=TRUE, y=TRUE required by simex
summary(Sip_mod)

# Update existing model to use errors-in-variable
Sip_eiv55=simex(Sip_mod, SIMEXvariable="lHoursEstimate", measurement.error=0.55) # valus from earlier analysis
summary(Sip_eiv55)

Sip_eiv60=simex(Sip_mod, SIMEXvariable="lHoursEstimate", measurement.error=0.60)
summary(Sip_eiv60)

